@@ -37,7 +37,7 @@ except ImportError:
     import configparser as ConfigParser
 
 
-__version__ = '0.18'
+__version__ = '0.19'
 __author__ = 'Marius Gedminas <marius@gedmin.as>'
 __licence__ = 'MIT'
 __url__ = 'https://github.com/mgedmin/check-manifest'
@@ -126,19 +126,21 @@ class CommandFailed(Failure):
                                command, status, output))
 
 
-def run(command):
+def run(command, encoding=None):
     """Run a command [cmd, arg1, arg2, ...].
 
     Returns the output (stdout + stderr).
 
     Raises CommandFailed in cases of error.
     """
+    if not encoding:
+        encoding = locale.getpreferredencoding()
     try:
         pipe = subprocess.Popen(command, stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT)
     except OSError as e:
         raise Failure("could not run %s: %s" % (command, e))
-    output = pipe.communicate()[0].decode(locale.getpreferredencoding(False))
+    output = pipe.communicate()[0].decode(encoding)
     status = pipe.wait()
     if status != 0:
         raise CommandFailed(command, status, output)
@@ -204,6 +206,17 @@ def get_one_file_in(dirname):
     return os.path.join(dirname, files[0])
 
 
+def unicodify(filename):
+    """Make sure filename is Unicode.
+
+    Because the tarfile module on Python 2 doesn't return Unicode.
+    """
+    if isinstance(filename, bytes):
+        return filename.decode(locale.getpreferredencoding())
+    else:
+        return filename
+
+
 def get_archive_file_list(archive_filename):
     """Return the list of files in an archive.
 
@@ -214,7 +227,7 @@ def get_archive_file_list(archive_filename):
             return add_directories(zf.namelist())
     elif archive_filename.endswith(('.tar.gz', '.tar.bz2', '.tar')):
         with closing(tarfile.open(archive_filename)) as tf:
-            return tf.getnames()
+            return add_directories(list(map(unicodify, tf.getnames())))
     else:
         ext = os.path.splitext(archive_filename)[-1]
         raise Failure('Unrecognized archive type: %s' % ext)
@@ -259,8 +272,11 @@ class Git(VCS):
     @staticmethod
     def get_versioned_files():
         """List all files versioned by git in the current directory."""
-        output = run(['git', 'ls-files'])
-        return add_directories(output.splitlines())
+        # Git for Windows uses UTF-8 instead of the locale encoding.
+        # Regular Git on sane POSIX systems uses the locale encoding
+        encoding = 'UTF-8' if sys.platform == 'win32' else None
+        output = run(['git', 'ls-files', '-z'], encoding=encoding)
+        return add_directories(output.split('\0')[:-1])
 
 
 class Mercurial(VCS):
@@ -364,9 +380,17 @@ WARN_ABOUT_FILES_IN_VCS = [
     'PKG-INFO',
     '*.egg-info',
     '*.mo',
+    '*.py[co]',
+    '*.so',
+    '*.pyd',
+    '*~',
+    '.*.sw[po]',
+    '.#*',
 ]
 
-SUGGESTIONS = [(re.compile(pattern), suggestion) for pattern, suggestion in [
+_sep = r'\\' if os.path.sep == '\\' else os.path.sep
+
+SUGGESTIONS = [(re.compile(pattern.replace('/', _sep)), suggestion) for pattern, suggestion in [
     # regexp -> suggestion
     ('^([^/]+[.](cfg|ini))$',       r'include \1'),
     ('^([.]travis[.]yml)$',         r'include \1'),
@@ -419,6 +443,21 @@ def read_manifest():
     IGNORE_REGEXPS.extend(ignore_regexps)
 
 
+def _glob_to_regexp(pat):
+    """Compile a glob pattern into a regexp.
+
+    We need to do this because fnmatch allows * to match /, which we
+    don't want.  E.g. an MANIFEST.in exclude of 'dirname/*css' should
+    match 'dirname/foo.css' but not 'dirname/subdir/bar.css'.
+    """
+    pat = fnmatch.translate(pat)
+    # Note that distutils in Python 2.6 has a buggy glob_to_re in
+    # distutils.filelist -- it converts '*.cfg' to '[^/]*cfg' instead
+    # of '[^\\]*cfg' on Windows.
+    sep = r'\\\\' if os.path.sep == '\\' else os.path.sep
+    return re.sub(r'((?<!\\)(\\\\)*)\.', r'\1[^%s]' % sep, pat)
+
+
 def _get_ignore_from_manifest(contents):
     """Gather the various ignore patterns from MANIFEST.in.
 
@@ -438,15 +477,11 @@ def _get_ignore_from_manifest(contents):
         if cmd == 'exclude':
             # An exclude of 'dirname/*css' can match 'dirname/foo.css'
             # but not 'dirname/subdir/bar.css'.  We need a regular
-            # expression for that.
+            # expression for that, since fnmatch doesn't pay attention to
+            # directory separators.
             for pat in rest.split():
-                if '*' in pat:
-                    pat = pat.replace('*', '[^/]*')
-                    # Do not make a dot into a magical wildcard character.
-                    pat = pat.replace('.', '\.')
-                    # XXX: we should handle other special glob and regexp characters,
-                    # i.e. ?, +, (, ), |, ^, $, {, }, \
-                    ignore_regexps.append(pat)
+                if '*' in pat or '?' in pat or '[!' in pat:
+                    ignore_regexps.append(_glob_to_regexp(pat))
                 else:
                     # No need for special handling.
                     ignore.append(pat)
@@ -480,7 +515,7 @@ def _get_ignore_from_manifest(contents):
             # not contain a path separator, as it actually has no
             # effect in that case, but that could differ per python
             # version.  We strip it here to avoid double separators.
-            rest = rest.rstrip(os.path.sep)
+            rest = rest.rstrip('/\\')
             ignore.append(rest)
             ignore.append(rest + os.path.sep + '*')
     return ignore, ignore_regexps
@@ -591,8 +626,9 @@ def check_manifest(source_tree='.', create=False, update=False,
                 info("suggested MANIFEST.in rules:\n%s"
                      % format_list(suggestions))
                 if user_asked_for_help:
+                    existed = os.path.exists('MANIFEST.in')
                     with open('MANIFEST.in', 'a') as f:
-                        if f.tell() == 0:
+                        if not existed:
                             info("creating MANIFEST.in")
                         else:
                             info("updating MANIFEST.in")
@@ -648,7 +684,7 @@ def main():
                               update=args.update, python=args.python):
             sys.exit(1)
     except Failure as e:
-        error(e)
+        error(str(e))
         sys.exit(2)
 
 
@@ -677,7 +713,7 @@ def zest_releaser_check(data):
                        " Do you want to continue despite that?", default=False):
                 sys.exit(1)
     except Failure as e:
-        error(e)
+        error(str(e))
         if not ask("Something bad happened. "
                    " Do you want to continue despite that?", default=False):
             sys.exit(2)
